#-*- Mode: Perl -*-

## File: WhoDat::Lex.pm
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description:
##  + WhoDat: stochastic lexicon
##======================================================================

package WhoDat::Lex;
use WhoDat::Object;
use WhoDat::Generator;
use Carp;

our @ISA = qw(WhoDat::Generator);


##======================================================================
## Structure
##  + object structure: (see WhoDat::Generator)
##     counts=>{''=>$total_count, $tag1=>{''=>$tagct, $tag1_tok1=>$count_1_1, ...}, ... }
sub new {
  my $that = shift;
  return $that->SUPER::new(@_);
}

##======================================================================
## I/O: Native

## $bool = $obj->saveNativeFh($fh,%args)
##  -- not implemented

## $obj = $class_or_obj->loadNativeFh($fh,%args)
sub loadNativeFh {
  my ($lx,$fh,%args) = @_;
  $lx = $lx->new(%args) if (!ref($lx));

  my ($line,$word,$count,@tagcts,$tag,$ct);
  while (defined($line=<$fh>)) {
    next if ($line =~ /^%%/);
    next if ($line =~ /^\s*$/);
    chomp($line);
    ($word,$count,@tagcts) = split(/\s+/, $line);

    ##-- ignore special tokens
    next if ($word =~ /^\@(?:[A-Z]+)$/ || $count <= 0);

    while (($tag,$ct) = splice(@tagcts,0,2)) {
      if (!defined($ct)) {
	carp(ref($lx), "::loadNativeFh(): no count for word '$word', tag '$tag'");
	next;
      }
      $lx->{counts}{$tag}{$word} += $ct;
      $lx->{counts}{$tag}{''}    += $ct;
      $lx->{counts}{''}          += $ct;
    }
  }
  return $lx;
}


1;

##======================================================================
## Docs
=pod

=head1 NAME

WhoDat::Lex - lexicon structure for WhoDat

=head1 SYNOPSIS

 use WhoDat::Lex;

 $lex = WhoDat::Lex->loadFile("mymodel.lex");
 $word = $lex->gen($tag);

=cut

##======================================================================
## Description
=pod

=head1 DESCRIPTION

WhoDat::Lex is a top-level class for stochastic lexical insertion.
A lexicon may be loaded from a text lexical frequency model as described
in the mootfiles(5) manpage.  WhoDat::Lex inherits from
WhoDat::Generator and WhoDat::Object.

=cut



##======================================================================
## Footer
=pod

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

=head1 AUTHOR

Bryan Jurish E<lt>jurish@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2005, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1),
mootfiles(5),
WhoDat(3perl),
WhoDat::Generator(3perl),
WhoDat::Object(3perl).

=cut
